import 'package:flutter/widgets.dart';

import 'mutation.dart';
import 'state_container.dart';
import 'state_container_context.dart';
import 'extensions.dart' show FrameworkOps;

extension SetStateAdapter on BuildContext {
  setState<T>(T value) {
    this.mutation(SetState(value));
  }
}

class StateChunk<T> {
  final T value;
  Type get key => T;

  StateChunk(this.value);

  query(ChunkedStateContainer ctr) {
    ctr.query<T>((ctrl, value, arg) => value[T]);
  }

  mutation(ChunkedStateContainer ctr) {
    ctr.mutation<SetState<T>>((ctrl, value, arg) {
      final newState = Map.from(value).cast<Type, dynamic>();
      newState[arg!.key] = arg.payload;
      return newState;
    });
  }
}

class ChunkedStateContainer extends StateContainer<Map<Type, dynamic>> {
  final List<StateChunk> chunks;
  final Stream initStream;

  ChunkedStateContainer(
    this.chunks,
    this.initStream,
  );

  @override
  Stream<Map<Type, dynamic>> init(StateContainerContext context) async* {
    yield* initStream.map((event) {
      final newState = Map.from(state).cast<Type, dynamic>();
      newState[event.runtimeType] = event;
      return newState;
    });
  }

  @override
  Map<Type, dynamic> get initialState => chunks.fold({}, (acc, el) {
        return {
          ...acc,
          el.key: el.value,
        };
      });

  @override
  queries(StateContainerContext context) {
    chunks.forEach((element) {
      element.query(this);
    });

    return super.queries(context);
  }

  @override
  mutations(StateContainerContext context) {
    chunks.forEach((element) {
      element.mutation(this);
    });

    return super.mutations(context);
  }
}
