import 'package:flutter/material.dart';
import 'package:microfrontends/microfrontends.dart';

import 'state_container.dart';

abstract class Dependency<T> {
  Effect<T> buildEffect(callback) {
    return Effect<T>(callback);
  }
}

class Subscription<T> extends Dependency<T> {}

class DynamicContainer<T extends StateContainer> extends Widget {
  final Widget? child;
  final List<Dependency> dependencies;
  final T container;

  const DynamicContainer({
    Key? key,
    required this.dependencies,
    required this.container,
    this.child,
  }) : super(key: key);

  @override
  Element createElement() {
    return DynamicContainerElement<T>(this);
  }
}

class DynamicContainerElement<T extends StateContainer>
    extends ComponentElement {
  DynamicContainerElement(Widget widget) : super(widget);

  String keyId = '$T(0)';
  int iteration = 0;

  @override
  void mount(Element? parent, Object? newSlot) {
    super.mount(parent, newSlot);

    widget.dependencies.forEach((element) {
      element.buildEffect((context, value) {
        // TODO: figure out smarter key generation
        keyId = '$T(${iteration++})';
        this.markNeedsBuild();
      }).bind(this);
    });
  }

  DynamicContainer<T> get widget => super.widget as DynamicContainer<T>;
  @override
  Widget build() {
    return KeyProvider<T>(
      keyId: keyId,
      child: widget.child,
      builder: (context, key, child) {
        return StaticContainer(
          key: key,
          container: widget.container,
          child: child!,
        );
      },
    );
  }
}
