import 'package:flutter/widgets.dart';
import 'package:microfrontends/src/context.dart';
import 'package:microfrontends/src/query.dart';

typedef EffectCallback<T> = void Function(BuildContext context, T value);

class Effect<T> {
  static final _context = Context<BuildContext, Effect>();

  static void dispatch<T>(BuildContext context, T value) {
    final effect = _context[context] as Effect<T>;
    effect._callback(context, value);
  }

  static deactivate(BuildContext context) {
    _context.remove(context);
  }

  final EffectCallback<T> _callback;
  late Query<T> _query;

  Effect(this._callback, [Object? queryParams]) {
    _query = Query<T>(queryParams);
  }

  void whenType<T>() {
    _query = _query.withUpdateCondition((state) {
      return state is T;
    });
  }

  void bind(BuildContext context) {
    _context[context] = this;
    _query.subscribe(context, offstage: true);
  }
}
