import 'package:flutter/material.dart';

typedef KeyProviderBuilder = Widget Function(
  BuildContext context,
  Key key,
  Widget? child,
);

class KeyProvider<T> extends Widget {
  final Object keyId;
  final KeyProviderBuilder builder;
  final Widget? child;

  String get scopedKey => '$T($keyId)';

  KeyProvider({
    Key? key,
    required this.keyId,
    required this.builder,
    this.child,
  }) : super(key: key);

  @override
  Element createElement() {
    return KeyProviderElement<T>(this);
  }
}

class KeyProviderElement<T> extends ComponentElement {
  KeyProviderElement(KeyProvider widget) : super(widget);

  KeyProvider get widget => super.widget as KeyProvider;

  @override
  void update(KeyProvider newWidget) {
    if (widget.scopedKey != newWidget.scopedKey) {
      markNeedsBuild();
    }

    super.update(newWidget);
  }

  @override
  Widget build() {
    final key = GlobalKey(debugLabel: widget.scopedKey);
    return widget.builder(this, key, widget.child);
  }
}
