import 'package:flutter/widgets.dart';

import 'resolver.dart';
import 'extensions.dart';

abstract class Mutation<T> {
  final T payload;

  Mutation(this.payload);

  void exec(BuildContext context) {
    final r = context.getMutationResolverOfType(runtimeType);
    final container = r.getContainer(context);
    final newState = r.resolve(container.stateNotifier, this);
    container.stateNotifier.value = newState;
    return newState;
  }
}

class MutationResolver<T extends Mutation, K> extends Resolver<K, K, T> {
  MutationResolver(
      ResolverController<K> ctrl, ResolverCallback<K, K, T> callback)
      : super(ctrl, callback);
}

class SetState<T> extends Mutation<T> {
  Type get key => T;
  SetState(T payload) : super(payload);
}
