import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';

import 'extensions.dart';

class ResolverController<T> {
  final ValueNotifier<T> _stateNotifier;

  ResolverController(this._stateNotifier);

  void run(Stream<T> Function() executor) {
    executor().forEach((element) {
      _stateNotifier.value = element;
    });
  }
}

typedef ResolverCallback<R, S, U> = R Function(
  ResolverController<S> ctrl,
  S value,
  U? arg,
);

class Resolver<P, T, U> {
  final ResolverController<P> _ctrl;
  final ResolverCallback<T, P, U> _callback;

  Resolver(this._ctrl, this._callback);

  T? resolve(ValueNotifier<P> stateNotifier, [dynamic args]) {
    return _callback(_ctrl, stateNotifier.value, args);
  }

  getContainer(BuildContext context) {
    return context.getContainerOfType<P>();
  }
}
