import 'package:flutter/widgets.dart';

import 'state_container.dart';
import 'extensions.dart';

final Map<Widget, StateContainer> _containersMap = {};

class StaticContainer<T> extends StatelessWidget {
  final Widget? child;
  final WidgetBuilder? builder;
  final StateContainer<T> container;

  const StaticContainer({
    Key? key,
    required this.container,
    this.child,
    this.builder,
  })  : assert(child != null || builder != null),
        super(key: key);

  StatelessElement createElement() {
    return StaticContainerElement(this);
  }

  @override
  Widget build(BuildContext context) {
    final c = _containersMap[this] ??= container;
    final child = this.child ?? Builder(builder: builder!);
    return child.withContainer<T>(context, c as StateContainer<T>);
  }
}

class StaticContainerElement<T> extends StatelessElement {
  StaticContainerElement(StatelessWidget widget) : super(widget);

  @override
  void update(covariant StatelessWidget newWidget) {
    final c = _containersMap.remove(widget)!;
    _containersMap[newWidget] = c;

    super.update(newWidget);
  }

  @override
  void unmount() {
    _containersMap.remove(widget);
    super.unmount();
  }
}
