import 'package:flutter/widgets.dart';

import 'query.dart';

typedef SubscriptionBuilder<T> = Widget Function(
  BuildContext context,
  T value,
  Widget? child,
);

typedef QueryResultBuilder<T> = Widget Function(
  BuildContext context,
  T data,
  Widget? child,
);

class QueryBuilder<T> extends Widget {
  final Query<T> query;
  final QueryResultBuilder<T> builder;
  final Widget? child;

  QueryBuilder({
    Key? key,
    required this.query,
    required this.builder,
    this.child,
  }) : super(key: key);

  @override
  Element createElement() {
    return QueryBuilderElement<T>(this);
  }
}

class QueryBuilderElement<T> extends ComponentElement {
  QueryBuilderElement(Widget widget) : super(widget);

  @override
  Widget build() {
    final w = widget as QueryBuilder<T>;

    final d = w.query.subscribe(this);
    return w.builder(this, d!, w.child);
  }

  @override
  void deactivate() {
    (widget as QueryBuilder).query.unsubscribe(this);
    super.deactivate();
  }
}
