import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:microfrontends/microfrontends.dart';

import 'mock/multitype_container.dart';

final groupTitle = 'Effect';

void main() {
  testWidgets(
      'registers effect callback which is being called when querand updates',
      (tester) async {
    final ctr = MultitypeContainer();
    MultitypeState? effectArg;

    await tester.pumpWidget(
      MaterialApp(
        home: StaticContainer(
          container: ctr,
          builder: (context) {
            Effect<MultitypeState>((context, state) {
              effectArg = state;
            }).bind(context);

            return SizedBox();
          },
        ),
      ),
    );

    ctr.state = TypeA(1);
    await tester.pump();

    expect(effectArg, equals(ctr.state));
  });

  testWidgets(
      '#whenType<T> registers effect callback which is being called when querand state is of exact type',
      (tester) async {
    final ctr = MultitypeContainer();
    MultitypeState? effectArg;

    await tester.pumpWidget(
      MaterialApp(
        home: StaticContainer(
          container: ctr,
          builder: (context) {
            Effect<MultitypeState>((context, state) {
              effectArg = state;
            })
              ..whenType<TypeB>()
              ..bind(context);

            return SizedBox();
          },
        ),
      ),
    );

    ctr.state = TypeA(1);
    tester.binding.scheduleFrame();
    await tester.pump();

    expect(effectArg, equals(null));

    ctr.state = TypeB(1);
    tester.binding.scheduleFrame();
    await tester.pump();

    expect(effectArg, equals(ctr.state));
  });
}
