import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:microfrontends/microfrontends.dart';

import 'mock/test_app.dart';

final groupTitle = 'KeyProvider';

void main() {
  testWidgets('triggers rebuild only if keyId is different', (tester) async {
    int keyId = 0;
    int buildsCount = 0;

    await tester.pumpWidget(
      TestApp(
        builder: (context) {
          return KeyProvider(
            keyId: keyId,
            builder: (context, key, _) {
              buildsCount++;
              return SizedBox();
            },
          );
        },
      ),
    );

    TestApp.rebuild();
    await tester.pump();
    expect(buildsCount, equals(1));

    TestApp.rebuild();
    await tester.pump();
    expect(buildsCount, equals(1));

    keyId = 1;
    TestApp.rebuild();
    await tester.pump();

    TestApp.rebuild();
    await tester.pump();
    expect(buildsCount, equals(2));
  });

  testWidgets('provides different keys on rebuild', (tester) async {
    Key? prevKey;
    int keyId = 0;

    await tester.pumpWidget(
      TestApp(
        builder: (context) {
          return KeyProvider(
            keyId: keyId,
            builder: (context, key, _) {
              expect(prevKey, isNot(equals(key)));
              prevKey = key;
              return SizedBox();
            },
          );
        },
      ),
    );

    keyId++;
    TestApp.rebuild();
    await tester.pump();

    keyId++;
    TestApp.rebuild();
    await tester.pump();
  });
}
